

# 服务bean
class ServiceBean:
  # 服务实现的包名，不同版本保持一致
  url = None
  # 名称
  name = None
  # 版本
  versionCode = None
  # 能力 K:能力id   V:能力信息
  capabilityBeanDict = {}
  
  # 事件 k:事件id  V:事件信息
  eventBeanDict = {}
  # 文件所在路径
  filePath = None
  def __init__(self) -> None:
    self.capabilityBeanDict = {}

  def __str__(self):
    capabilityStr = "{"
    for key, value in self.capabilityBeanDict.items():
      capabilityStr += "\n \t\t "+value.__str__()
    eventBeanStr = "{"
    for key, value in self.eventBeanDict.items():
      eventBeanStr += "\n \t\t "+value.__str__()
    return f"\nServiceBean(filePath={self.filePath},url={self.url}, name={self.name}, versionCode={self.versionCode}, \n \t capabilityBeanDict={capabilityStr} ,\n \t eventBeanDict = {eventBeanStr}) \n"
 


from .. import applicationContext
from ... import config
from .baseSender import BaseSender
import json

class HardwareUpLoadSender(BaseSender):

  def send():
    hardwareInfo = applicationContext.hardwareInfoLoader.getHardwareInfo()
    applicationContext.pervasiveMqttClient.sendMsg(config.UPLOAD_HARDWARE_TOPIC,json.dumps(hardwareInfo, ensure_ascii=False))


from ..pojo.eventBean import EventBean

'''
* 创建一个系统事件
*
'''
def createSystemEvent(eventId:str,topic:str)->EventBean:
    event = EventBean()
    event.eventId = eventId
    event.attributes = {         
                    "host":"",
                    "topic":topic,
                    "username":"",
                    "password":""}
    return event
    


from ..scheduleService import ScheduleBase
from .. import applicationContext


class MqttHealthJob(ScheduleBase):

  """
  * 发送心跳包
  """
  def handleInterrupt(self):
    print("beat")
    # applicationContext.pervasiveMqttClient.ping()


import gc
import os
import time
import tarfile

class TarGzUtils:
    def __init__(self, filePath):
        # 检查文件名是否以.tar.gz结尾
        if not filePath.endswith(".tar.gz"):
            raise ValueError("Invalid filename: {}".format(filePath))
        self.filePath = filePath
    
    def extractToDir(self, dirPath):
        try:
            with tarfile.open(self.filePath, "r:gz") as tar:
                tar.extractall(path=dirPath)
        except tarfile.TarError as e:
            raise Exception("解压异常 {}: {}".format(self.filePath, str(e)))
        pass
    
    def extractAndDeleteToDir(self, dir):
        try :
            # 解压并删除.tar.gz文件到指定目录
            self.extractToDir(dir)
        finally :
            # 删除原始文件
            os.remove(self.filePath) 



from ..scheduleService import ScheduleBase
from ...proxy.thingProxy import thingProxy
from .. import applicationContext
 
class EventListenerJob(ScheduleBase):
  
  """
  * 监听事件
  """
  def handleInterrupt(self):
    thingModel = thingProxy({"operation":"read"}).handle(None)
    print(thingModel)
    for event in applicationContext.events:
        applicationContext.pervasiveMqttClient.sendMsg("/service/event/test","test")
#     print(self.events)




#字典深拷贝 注意字典中不能有Object
def dictCopy(oldDict):
  newDick = {}
  list = oldDict.items()
  for k,v in list:
    if isinstance(v,dict):
      newDick[k] = dictCopy(v)
    else:
      newDick[k] = v 
  return newDick


from iotPervasiveServiceSDK.core.loader import thingProxyFuncLoader
import json


'''
物模型属性读写代理
'''
class thingProxy():
  
  initdata = None
  def __init__(self,initdata) -> None:
    self.initdata = initdata
    pass

  def handle(self,request):
    if(self.initdata["operation"]=="read"):
      return thingProxyFuncLoader.readFunc()
    elif(self.initdata["operation"]=="writer"):
      request=json.loads(request)
      return thingProxyFuncLoader.writerFunc(request)
    else:
      raise Exception("no operation is"+ self.initdata["operation"])



# 能力bean

class CapabilityBean:
  # 能力id
  capabilityId = None
  # 代理类型
  proxyType = None
  # 参数
  attributes = {}
  # 输入模板的语言
  inputLanguage = None
  # 输入模板
  inputMapper = None
  # 输出模板的语言
  outputLanguage = None
  # 输出模板
  outputMapper = None

  # 定义__str__方法，返回所有属性和值的字符串表示
  def __init__(self) -> None:
    self.attributes = {}

  def __str__(self):
    return f"CapabilityBean(capabilityId={self.capabilityId}, proxyType={self.proxyType}, attributes={self.attributes}, inputLanguage={self.inputLanguage}, inputMapper={self.inputMapper}, outputLanguage={self.outputLanguage}, outPutMapper={self.outputMapper})"



import copy
import json
from ..core import applicationContext
from ..utils import pathUtils as PathUtil

from .exception.serviceException import ServiceNotExistException
from .exception.serviceException import CapabilityNotExistException
from .exception.serviceException import ProxyNotExistException
from .exception.serviceException import InvokeServiceException
from .exception.commonException import invalidParamsException
from ..model.ujinja.source import TemplateEngine
from ..proxy.thingProxy import thingProxy

class ServerInvoker:
  '''
  * 调用服务
  * serviceId  服务id
  * capabilityId 能力id
  * postJsonDict 消息的参数字典
  '''
  def invokeService(serviceId:str,capabilityId:str,requestJsonDict:dict):
    requestJsonDict["system"] = applicationContext.getSystemInfo()
    # 特判物模型读写服务    
    if (serviceId == "thingService"):
        return thingProxy({"operation":capabilityId}).handle(requestJsonDict)
    # 从服务字典读取服务信息
    if (serviceId==None):
        raise invalidParamsException("id",serviceId)
    if (capabilityId == None):
        raise invalidParamsException("capabilityId",capabilityId)
    service = applicationContext.serviceDict.get(serviceId,None)
    if (service == None):
        raise ServiceNotExistException(serviceId)
    capability = service.capabilityBeanDict.get(capabilityId,None)
    if (capability==None):
        raise CapabilityNotExistException(capabilityId)
    # jinja2 输入消息转换
    runData = None
    try:
      if(capability.inputMapper != None):
        if(capability.inputLanguage == "jinja2"):
          inputTemplateEngine = TemplateEngine()
          inputTemplateEngine.load_template(capability.inputMapper)
          runData = inputTemplateEngine.render_template(requestJsonDict)
          print(runData)
    except Exception as e:
      raise InvokeServiceException("服务定义错误,输入消息映射错误,请联系服务开发者")
        
    initData =  copy.deepcopy(capability.attributes)
    # 如果是python代理则需要加载脚本所在位置
    if(capability.proxyType == "pythonServiceProxy"):
      scriptPath = PathUtil.joinPath(service.filePath,"script")
      initData["script"] = PathUtil.joinPath(scriptPath,initData["script"])
    response = ServerInvoker.invokeProxy(capability.proxyType,initData,runData)
    #输出消息转换
    if(capability.outputMapper != None):
      if(capability.outputLanguage == "jinja2"):
        outputTemplateEngine = TemplateEngine()
        outputTemplateEngine.load_template(capability.outputMapper)
        try:
          if(isinstance(response, str)):
            response = json.loads(response)
          response["system"] = applicationContext.getSystemInfo()
          response = outputTemplateEngine.render_template(response)
          print(type(response))
          print(response)
          if(isinstance(response, str)):
            response = json.loads(response)
          print(type(response))
          print(response)
        except Exception as e:
          raise InvokeServiceException(json.dumps({
            "response":response,
            "systemInfo":"服务定义错误,输出消息映射错误,请联系服务开发者"+str(e)
            }, ensure_ascii=False))
          
        print(response)
    return response


  '''
  * 运行代理
  * proxyName 代理名称
  * initData  初始化数据
  * runData   运行时数据
  '''
  def invokeProxy(proxyName:str,initData:any,runData):
      proxyModel = applicationContext.proxyDict.get(proxyName,None)
      if(proxyModel==None):
          raise ProxyNotExistException(proxyName)
      if(hasattr(proxyModel,proxyName)):
        proxyClass = getattr(proxyModel,proxyName)
        proxy = proxyClass(initData)
        return proxy.handle(runData)
  
    


import _thread
from time import sleep


class TaskUtil:
    """创建新线程并循环运行指定函数"""

    def __init__(self, func=lambda: None, sec=-1, *args, **kwargs):
        """
        * func 需要循环运行的函数
        * sec  每次循环的延迟，负数则只运行一次
        * args kwagrs 函数的参数
        * enable 使能运行
        """
        self.sec = sec
        self.func = func
        self.args, self.kwargs = args, kwargs
        self.enable = True
        self.lock = _thread.allocate_lock()
        self.stop_lock = _thread.allocate_lock()
        self.lock.acquire()
        self.stop_lock.acquire()
        _thread.start_new_thread(self.__run, ())

    def __run(self):
        """
        # 请勿调用
        * 线程运行函数
        """
        while True:
            self.lock.acquire()
            try:
                self.func(*self.args, **self.kwargs)
            except Exception as e:
                print('Task_function_error:', e)
            if self.sec < 0 or not self.enable:
                self.stop_lock.release()
            else:
                sleep(self.sec)
                self.lock.release()

    def start(self):
        """运行线程"""
        self.lock.release()

    def stop(self):
        """暂停线程"""
        self.enable = False
        self.stop_lock.acquire()
        self.enable = True


class ServiceException(Exception):
    def __init__(self, msg):
        self.message = f"service exception: {msg}"
        super().__init__(self.message)


    def __str__(self):
        return repr(self.message)


# 加载服务时异常
class LoadingServiceException(ServiceException):
    def __init__(self, msg):
        self.message = f"An error occurred loading the service: {msg}"
        super().__init__(self.message)
    def __str__(self):
        return repr(self.message)
 

# 调用服务时异常
class InvokeServiceException(ServiceException):
    def __init__(self, msg):
        self.message = f"Invoke Service Exception: {msg}"
        super().__init__(self.message)
    
    def __str__(self):
        return repr(self.message)


# 没有找到路径异常
class LoadPathNotfoundException(LoadingServiceException):
    def __init__(self, path):
        self.message = f"The path '{path}' does not exist"
        super().__init__(self.message)

    def __str__(self):
        return repr(self.message)
    
# 服务不存在异常  
class ServiceNotExistException(InvokeServiceException):
    def __init__(self, serviceId):
        self.message = f"There is no service with serviceId = {serviceId}"
        super().__init__(self.message)

    def __str__(self):
        return repr(self.message)
    
    
# 能力不存在异常  
class CapabilityNotExistException(InvokeServiceException):
    def __init__(self, capabilityId,serviceId):
        self.message = f"There is no capability with capabilityId = {capabilityId},serviceId = {serviceId}"
        super().__init__(self.message)

    def __str__(self):
        return repr(self.message)

# 事件不存在异常  
class EventNotExistException(InvokeServiceException):
    def __init__(self, eventId,serviceId):
        self.message = f"There is no event with eventId = {eventId},serviceId = {serviceId}"
        super().__init__(self.message)

    def __str__(self):
        return repr(self.message)

# 代理不存在异常  
class ProxyNotExistException(InvokeServiceException):
    def __init__(self, proxy):
        self.message = f"There is no proxy with  proxy = {proxy}"
        super().__init__(self.message)

    def __str__(self):
        return repr(self.message)




from ...utils import logutil

log = logutil.get_logger(__name__)

class HardwareInfoLoader:

  
  def __init__(self):
    # 存储用户提供的静态硬件信息
    self.staticHardwareData = {}
    # 存储获取实时硬件信息的回调函数
    self.dynamicHardwareCallbacks = {}  

  """
  设置静态硬件信息。

  参数:
      dataDict (dict): 包含用户自定义静态硬件信息的字典。
  """
  def setStaticHardwareData(self, dataDict):
    self.staticHardwareData = dataDict  


  """
  注册获取实时硬件信息的回调函数。

  参数:
      key (str): 用户定义的硬件信息的键（key）。
      callbackFunc (function): 回调函数，用于获取实时硬件信息。
  """
  def registerDynamicHardwareCallback(self, key, callbackFunc):
    self.dynamicHardwareCallbacks[key] = callbackFunc

  """
  获取用户定义的硬件信息。

  返回:
      dict: 包含用户定义的硬件信息。
  """
  def getHardwareInfo(self):

    hardwareInfo = self.staticHardwareData.copy()

    # 获取实时硬件信息
    for key, callback in self.dynamicHardwareCallbacks.items():
      try:
        hardwareInfo[key] = callback()
      except Exception as e:
        log.error("动态获取硬件信息异常，请检查HardwareInfo回调"+key+":"+str(e))

    return hardwareInfo


from iotPervasiveServiceSDK.proxy.proxyBase import ProxyBase 
import os
import importlib

'''
python服务代理
'''
class pythonServiceProxy(ProxyBase):
  script = None

  '''
  python服务代理初始化
  '''
  def __init__(self,initData) -> None:
    print("====pythonServiceProxy====")
    file_name = os.path.basename(initData["script"])
    file_name = os.path.splitext(file_name)[0]
    spec = importlib.util.spec_from_file_location(file_name, initData["script"])

    module = importlib.util.module_from_spec(spec)

    spec.loader.exec_module(module)

    self.script = module
    


  def handle(self,request):
    return self.script.execute(request)
    


from . import applicationContext
from .. import config
from ..pojo.serviceBean import ServiceBean
from ..pojo.eventBean import EventBean
from .exception.serviceException import ServiceNotExistException
from .exception.serviceException import EventNotExistException
from ..model.ujinja.source import TemplateEngine
from ..systemService.mqttService import MqttService
import json
import os

'''
* 事件调用类
'''
class EventInvoker:

  '''
  *调用系统事件
  '''
  @staticmethod
  def invokeSystemEvent(eventId:str,eventArgs:dict = {}):
    EventInvoker.invokeEvent("systemService",eventId,eventArgs)


  '''
  * 调用事件
  * serviceId 服务id
  * eventId 事件id
  * eventArgs 事件参数字典
  '''
  @staticmethod
  def invokeEvent(serviceId:str,eventId:str,eventArgs:dict = {}):
    eventArgs["system"] = applicationContext.getSystemInfo()
    
    service:ServiceBean = applicationContext.serviceDict.get(serviceId,None)
    if service == None:
      raise ServiceNotExistException(serviceId)
    
    event:EventBean = service.eventBeanDict.get(eventId,None)
    if event == None:
      raise EventNotExistException(eventId,serviceId)
    
    eventMsg = None
    if event.messageMapper!=None:
      if(event.messageLanguage == "jinja2" or event.messageLanguage == None):
          inputTemplateEngine = TemplateEngine()
          inputTemplateEngine.load_template(event.messageMapper)
          eventMsg = inputTemplateEngine.render_template(eventArgs)
          print("== event msg Mapper == ")
          print(eventMsg)

    # 默认是mqtt的方式上报topic
    if event.protocolType == "MQTT" or event.protocolType == None:
      EventInvoker._MQTTEventSender(service,event,eventMsg)


  '''
  * mqtt事件发送
  '''
  @staticmethod
  def _MQTTEventSender(service:ServiceBean,event:EventBean,eventMsg:str):
    host:str = event.attributes.get("host",None)
    isSystemMqtt = False
    mqttClient = None

    # 默认使用系统的mqtt客户端
    if host == "" or host == None:
      isSystemMqtt = True
      mqttClient = applicationContext.pervasiveMqttClient
    else:
      addressList:list = host.split(":")
      addressList[1] = int(addressList[1])
      if addressList[0] == config.MQTT_ADDRESS and addressList[1] == config.MQTT_PORT:
        isSystemMqtt = True
        mqttClient = applicationContext.pervasiveMqttClient
      else:
        mqttClient = MqttService(''.join([('0'+hex(ord(os.urandom(1)))[2:])[-2:] for x in range(8)]),addressList[0],addressList[1])

    topic = event.attributes.get("topic",config.ON_EVENT_TOPIC)
    msgJsonDict = {
      "deviceInstanceId":config.DEVICE_ID,
      "serviceUrl": service.url,
      "versionCode": service.versionCode,
      "eventId":event.eventId,
      "msg": eventMsg
    }

    mqttClient.sendMsg(topic,json.dumps(msgJsonDict, ensure_ascii=False))
    # 如果非系统的mqtt客户端则关闭
    if not isSystemMqtt:
      mqttClient.close()


  def getEventMsgStr(service:ServiceBean,event:EventBean,eventMsg:str):
    msgJsonDict = {
      "deviceInstanceId":config.DEVICE_ID,
      "serviceUrl": service.url,
      "versionCode": service.versionCode,
      "eventId":event.eventId,
      "msg": eventMsg
    }
    return json.dumps(msgJsonDict, ensure_ascii=False)
    


  
  

    



import os
from ...utils import logutil
from ...pojo.serviceBean import ServiceBean
from ...pojo.capabilityBean import CapabilityBean
from ...pojo.eventBean import EventBean
from ..exception.serviceException import LoadingServiceException
from ..exception.serviceException import LoadPathNotfoundException
import os
from ... import config
from .. import applicationContext
import json
import sys
from ...factory.systemServiceFactory import SystemServiceFactory
from ...utils import pathUtils as PathUtil


log = logutil.get_logger(__name__)
'''
服务加载工具
'''
class ServiceLoader:

  '''
  加载服务字典
  input：
    - servicePath = 服务的相对路径（相对于项目根目录的）
    - BASE_PATH = 基本路径（设备中mian.py所在文件夹的绝对路径）
  '''
  def LoadServiceDict(servicePath,BASE_PATH):
    applicationContext.events = []
    serviceDict = {}
    absolutePath=PathUtil.joinPath(BASE_PATH,servicePath)
    # 遍历文件
    serviceDict = ServiceLoader.dfsDir(absolutePath,serviceDict)
    systemServer = ServiceLoader.loadSystemservice()
    serviceDict[systemServer.url] = systemServer
    str = ""
    for key, value in serviceDict.items():
      str+=value.__str__()
    log.info(str)
    return serviceDict

  '''
   递归扫描文件
  '''
  def dfsDir(path,serviceDict):
    log.debug(path)
    try:
      listdir=os.listdir(path)
    except :
      log.warning("服务文件夹不存在"+path)
      return {}
    for item in listdir:
      try: 
          item_path = PathUtil.joinPath(path, item)
          # 如果是目录             
          if os.path.isdir(item_path):
              # 忽略script文件夹
              if item == "script":
                  continue
              ServiceLoader.dfsDir(item_path,serviceDict)
          # 是文件             
          else:
            try:
              if item.find("config.json") >= 0:
                  ServiceLoader.loadingConfigJson(path,item,serviceDict)
            except LoadingServiceException as e:
              log.error(repr(e.value))
            except  Exception as e:
              log.error(repr(e.value))
      except StopIteration as e:
          break
    return serviceDict

  # 读取configJson文件并作解析，将解析后的结果存入字典
  def loadingConfigJson(serviceAbsPath,jsonName,serviceDict):
    jsonPath = PathUtil.joinPath(serviceAbsPath,jsonName)
    file = open(jsonPath,"r", encoding='utf-8')
    jsonString = file.read()
    file.close()
    jsonBean = json.loads(jsonString)
    serviceBean = ServiceLoader.loadingServiceInfo(jsonBean,serviceAbsPath)
    serviceDict[serviceBean.url] = serviceBean


  '''
  * 加载服务信息
  * jsonDict json解析后的字典
  * filePath 文件所在路径
  '''
  def loadingServiceInfo(jsonDict:dict,filePath:str)->ServiceBean:
    # 获取基本信息    
    identifier = jsonDict.get("identifier",None)
    if identifier == None:
      # todo 抛出异常
      log.error("identifier == None")
      return
    serviceBean = ServiceBean()
    serviceBean.url = identifier.get("url",None)
    serviceBean.versionCode = identifier.get("versionCode",None)
    if serviceBean.url==None or serviceBean.versionCode==None:
      # todo 抛出异常
      log.error("serviceBean.url==None or serviceBean.versionCode==None")
      return
    serviceBean.filePath = filePath
    serviceBean.name = identifier.get("name",None)
    try:
      # 解析其它文件信息
      metaData = jsonDict.get("meta-data",None)
      if metaData != None:
        capabilitiesFileName = metaData.get("capabilities",None)
        if capabilitiesFileName!=None:
          capabilitiesFilePath = PathUtil.joinPath(filePath,capabilitiesFileName)
          serviceBean.capabilityBeanDict=ServiceLoader.loadingCapabilitys(capabilitiesFilePath)
        eventsFileName = metaData.get("events",None)
        if eventsFileName != None:
          eventsFilePath = PathUtil.joinPath(filePath,eventsFileName)
          serviceBean.eventBeanDict = ServiceLoader.loadingEvents(eventsFilePath)
    except LoadingServiceException as e:
      raise LoadingServiceException("ServiceImplId(url) is "+serviceBean.url+str(e))
    except Exception as e:
      raise LoadingServiceException("unknown error : ServiceImplId(url) is "+serviceBean.url)



    return serviceBean
    
  '''
  * 加载能力列表
  '''
  def loadingCapabilitys(capabilitiesFilePath:str)->dict:
    res = {}
    try:
      capabilitiesFile = open(capabilitiesFilePath,"r", encoding='utf-8')
      capabilitiesJsonString = capabilitiesFile.read()
      capabilitiesFile.close()
      capabilitiesJsonArray = json.loads(capabilitiesJsonString)["capabilities"]["capability"]
    except KeyError as e:
      # sys.print_exception(e)
      raise LoadingServiceException("keyError: event Json loadFail: Check whether the events and event tags exist in the json format ")
    except OSError as e:
      # sys.print_exception(e)
      raise LoadingServiceException("OsError: The file does not exist: Check whether the service pack is faulty,path:" + capabilitiesFilePath)
    for item in capabilitiesJsonArray:
      try:
        capabilityBean = CapabilityBean()
        capabilityBean.capabilityId = item.get("id",None)
        capabilityBean.proxyType = item.get("proxy",None)
        capabilityBean.attributes = item.get("attributes",None)
        # capabilityBean.inputLanguage = item[inputMapper]
        
        if(item.get("inputMapper",None)!=None):
          mapperInfo = item.get("inputMapper",None)
          capabilityBean.inputLanguage = mapperInfo.get("templateLanguage","jinja2")
          capabilityBean.inputMapper = mapperInfo["templateText"]
        if(item.get("outputMapper",None)!=None):
          mapperInfo = item.get("outputMapper",None)
          capabilityBean.outputLanguage = mapperInfo.get("templateLanguage","jinja2")
          capabilityBean.outputMapper = mapperInfo["templateText"]
        res[capabilityBean.capabilityId] = capabilityBean
      except Exception as e:
        log.error(repr(e.value))
        if(capabilityBean.capabilityId!=None):
          raise LoadingServiceException("capability resolution error: occurred at the location of capability id "+ capabilityBean.capabilityId)
        else:
          raise LoadingServiceException("capability resolution error: The capability id does not exist")



    return res

  def loadingEvents(eventsFilePath:str)->dict:
    res = {}
    try :
      eventsFile = open(eventsFilePath,"r", encoding='utf-8')
      eventsFileJsonStr = eventsFile.read()
      eventsFile.close()  
      eventsFileJsonArray = json.loads(eventsFileJsonStr)["events"]["event"]
    except KeyError as e:
      # sys.print_exception(e)
      raise LoadingServiceException("keyError: event Json loadFail: Check whether the events and event tags exist in the json format ")
    except OSError as e:
      # sys.print_exception(e)
      raise LoadingServiceException("OsError: The file does not exist: Check whether the service pack is faulty,path:" + eventsFilePath)
    for item in eventsFileJsonArray:
      try:
        eventBean = EventBean()
        eventBean.eventId = item.get("id",None)
        eventBean.attributes = item["attributes"]
        if(item.get("messageMapper",None)!=None):
          mapperInfo = item.get("messageMapper",None)
          eventBean.messageLanguage = mapperInfo.get("templateLanguage","jinja2")
          eventBean.messageMapper = mapperInfo["templateText"]
        res[eventBean.eventId] = eventBean
      except Exception as e:
        log.error(repr(e.value))
        if(eventBean.eventId!=None):
          raise LoadingServiceException("Event resolution error: occurred at the location of event id "+ eventBean.eventId)
        else:
          raise LoadingServiceException("Event resolution error: The event id does not exist")
    return res


  '''
  * 系统自带事件加载
  '''
  def loadSystemservice() -> ServiceBean:
    
    return SystemServiceFactory.loadSystemService()
    

    
    








from ..scheduleService import ScheduleBase
from .. import applicationContext

class MqttCheckMessageJob(ScheduleBase):

  """
  * 中断处理 接收mqtt消息
  """
  def handleInterrupt(self):
    applicationContext.pervasiveMqttClient.check_msg() 



    


'''
* 模板转换
* input: template:模板的字符串   data:替换内容的字典
* output：拼接好的模板
* 
'''
def templateConversion(template: str, data: dict) -> str:
    # 获取字典中的值
    def get_value(key: str):
        keys = key.split('.')
        value = data
        for k in keys:
            value = value[k]
        if isinstance (value,bool):
            value = str(value).lower()
        return value

    result = template
    # 获取标签位置
    start_index = result.find('{{')
    while start_index != -1:
        end_index = result.find('}}', start_index)
        #获取到key
        key = result[start_index + 2:end_index].strip()
        #根据key获取value
        value = get_value(key)
        #拼接
        result = result[:start_index] + str(value) + result[end_index + 2:]
        #获取下一个标签位置
        start_index = result.find('{{')
    return result